#include <cstdio>
#include <iostream>
#include <memory>

#ifdef WIN32
#include <windows.h> //for Sleep()
#endif

using std::auto_ptr;

#ifndef RTI_USES_STD_FSTREAM
#define RTI_USES_STD_FSTREAM
#endif
#include <RTI.hh>
#include <NullFederateAmbassador.hh>
#include <Enums.h>

RTI::ObjectClassHandle	base_id;
RTI::ObjectClassHandle	aircraft_id;
//This application does not know about submarine_id

RTI::ObjectHandle		object_id;

//BaseEntity attrs:
RTI::AttributeHandle	attr_location_id;
RTI::AttributeHandle	attr_orientation_id;
RTI::AttributeHandle	attr_velocity_id;

//Aircraft attrs:
RTI::AttributeHandle	attr_afterburner_id;


class MyFederate: public NullFederateAmbassador
{
	public:
		MyFederate(){};

		void discoverObjectInstance(RTI::ObjectHandle theObject,RTI::ObjectClassHandle theObjectClass,const char *theObjectName) throw ( RTI::CouldNotDiscover,RTI::ObjectClassNotKnown,RTI::FederateInternalError)
		{
			if (theObjectClass==base_id)
				printf("Found BaseEntity (tag=%s) class=%i id=%i\n",theObjectName,theObjectClass,theObject);
			else if (theObjectClass==aircraft_id)
				printf("Found Aircraft (tag=%s) class=%i id=%i\n",theObjectName,theObjectClass,theObject);
			else
				printf("Unknown object found (tag=%s) class=%i id=%i\n",theObjectName,theObjectClass,theObject);
		}

		void reflectAttributeValues(RTI::ObjectHandle theObject,const RTI::AttributeHandleValuePairSet &theAttributes,const char *theTag) throw ( RTI::ObjectNotKnown,RTI::AttributeNotKnown,RTI::FederateOwnsAttributes,RTI::FederateInternalError){}
		
		void removeObjectInstance(RTI::ObjectHandle theObject, const char *tag) throw (RTI::ObjectNotKnown, RTI::FederateInternalError)
		{
			printf("Removing Object (tag=%s) id=%i\n",tag,theObject);
		}
		void receiveInteraction(RTI::InteractionClassHandle theInteraction, const RTI::ParameterHandleValuePairSet &theData, const char *theTag) throw (RTI::InteractionClassNotKnown, RTI::InteractionParameterNotKnown, RTI::FederateInternalError){}
};

void Snooze(float secs)
{
#ifdef WIN32
	Sleep((int)(secs*1000));
#else
	sleep(secs);
#endif
}

RTI::RTIambassador		*rtiAmb;
MyFederate				*myfed;


int main()
{
	try
	{
		rtiAmb=new RTI::RTIambassador();
		myfed=new MyFederate();
	}
	catch(RTI::Exception &e)
	{
		printf("%s\n",e._name);
		Snooze(2.0f);
		exit(1);
	}
	
	try
	{
		rtiAmb->createFederationExecution("Recon", "recon.fed");
	}
	catch(RTI::FederationExecutionAlreadyExists &e)
	{
		e._name; //ignore this 'exceptional circumstance' of the federation already existing...
	}
	catch(RTI::Exception &e)
	{
		printf("%s\n",e._name);
		Snooze(2.0f);
		exit(1);
	}

	try
	{
		rtiAmb->joinFederationExecution("App1","Recon",myfed);
	}
	catch(RTI::Exception &e)
	{
		printf("%s\n",e._name);
		Snooze(2.0f);
		exit(1);
	}

	base_id = rtiAmb->getObjectClassHandle("BaseEntity");
	aircraft_id = rtiAmb->getObjectClassHandle("Aircraft");

	//base entity attrs
	std::auto_ptr<RTI::AttributeHandleSet> attrs_base(RTI::AttributeHandleSetFactory::create(3));
	attrs_base->add(attr_location_id = rtiAmb->getAttributeHandle("WorldLocation", base_id));
	attrs_base->add(attr_orientation_id = rtiAmb->getAttributeHandle("Orientation", base_id));
	attrs_base->add(attr_velocity_id = rtiAmb->getAttributeHandle("VelocityVector", base_id));
	rtiAmb->subscribeObjectClassAttributes(base_id, *attrs_base);
	
	//aircraft entity attrs
	std::auto_ptr<RTI::AttributeHandleSet> attrs_ac(RTI::AttributeHandleSetFactory::create(1));
	attrs_ac->add(attr_afterburner_id = rtiAmb->getAttributeHandle("AfterburnerOn", aircraft_id));
	rtiAmb->subscribeObjectClassAttributes(aircraft_id, *attrs_ac);
	
	printf("Application1: Configured ok now running\n");
	//now do nothing and wait
	for(;;)
	{
		Snooze(1.0f);
		try
		{
			rtiAmb->tick();
		}
		catch(RTI::Exception &e)
		{
			printf("%s\n",e._name);
			Snooze(2.0f);
			exit(1);
		}

	}

	
}
